@@ -49,6 +49,7 @@ class Config:
     debug = False
     dogstatsd_tags = ""
     errorlog: Union[logging.Logger, str, None] = "-"
+    graceful_timeout: float = 3 * SECONDS
     group: Optional[int] = None
     h11_max_incomplete_size = 16 * 1024 * BYTES
     h2_max_concurrent_streams = 100
@@ -64,6 +65,7 @@ class Config:
     max_app_queue_size: int = 10
     pid_path: Optional[str] = None
     root_path = ""
+    server_names: List[str] = []
     shutdown_timeout = 60 * SECONDS
     ssl_handshake_timeout = 60 * SECONDS
     startup_timeout = 60 * SECONDS
@@ -75,6 +77,7 @@ class Config:
     verify_flags: Optional[VerifyFlags] = None
     verify_mode: Optional[VerifyMode] = None
     websocket_max_message_size = 16 * 1024 * 1024 * BYTES
+    websocket_ping_interval: Optional[int] = None
     worker_class = "asyncio"
     workers = 1
 
