@@ -5,7 +5,8 @@ from urllib.parse import unquote
 
 from .events import Body, EndBody, Event, Request, Response, StreamClosed
 from ..config import Config
-from ..utils import build_and_validate_headers, suppress_body, UnexpectedMessage
+from ..typing import ASGIFramework, Context
+from ..utils import build_and_validate_headers, suppress_body, UnexpectedMessage, valid_server_name
 
 PUSH_VERSIONS = {"2", "3"}
 
@@ -22,23 +23,25 @@ class ASGIHTTPState(Enum):
 class HTTPStream:
     def __init__(
         self,
+        app: ASGIFramework,
         config: Config,
+        context: Context,
         ssl: bool,
         client: Optional[Tuple[str, int]],
         server: Optional[Tuple[str, int]],
         send: Callable[[Event], Awaitable[None]],
-        spawn_app: Callable[[dict, Callable], Awaitable[Callable]],
         stream_id: int,
     ) -> None:
+        self.app = app
         self.client = client
         self.closed = False
         self.config = config
+        self.context = context
         self.response: dict
         self.scope: dict
         self.send = send
         self.scheme = "https" if ssl else "http"
         self.server = server
-        self.spawn_app = spawn_app
         self.start_time: float
         self.state = ASGIHTTPState.REQUEST
         self.stream_id = stream_id
@@ -48,7 +51,10 @@ class HTTPStream:
         return False
 
     async def handle(self, event: Event) -> None:
-        if isinstance(event, Request):
+        if self.closed:
+            return
+        elif isinstance(event, Request):
+            self.start_time = time()
             path, _, query_string = event.raw_path.partition(b"?")
             self.scope = {
                 "type": "http",
@@ -66,15 +72,22 @@ class HTTPStream:
             }
             if event.http_version in PUSH_VERSIONS:
                 self.scope["extensions"] = {"http.response.push": {}}
-            self.start_time = time()
-            self.app_put = await self.spawn_app(self.scope, self.app_send)
+
+            if valid_server_name(self.config, event):
+                self.app_put = await self.context.spawn_app(
+                    self.app, self.config, self.scope, self.app_send
+                )
+            else:
+                await self._send_error_response(404)
+                self.closed = True
+
         elif isinstance(event, Body):
             await self.app_put(
                 {"type": "http.request", "body": bytes(event.data), "more_body": True}
             )
         elif isinstance(event, EndBody):
             await self.app_put({"type": "http.request", "body": b"", "more_body": False})
-        elif isinstance(event, StreamClosed) and not self.closed:
+        elif isinstance(event, StreamClosed):
             self.closed = True
             if self.app_put is not None:
                 await self.app_put({"type": "http.disconnect"})
@@ -87,18 +100,7 @@ class HTTPStream:
         if message is None:  # ASGI App has finished sending messages
             # Cleanup if required
             if self.state == ASGIHTTPState.REQUEST:
-                await self.send(
-                    Response(
-                        stream_id=self.stream_id,
-                        headers=[(b"content-length", b"0"), (b"connection", b"close")],
-                        status_code=500,
-                    )
-                )
-                await self.send(EndBody(stream_id=self.stream_id))
-                self.state = ASGIHTTPState.CLOSED
-                await self.config.log.access(
-                    self.scope, {"status": 500, "headers": []}, time() - self.start_time
-                )
+                await self._send_error_response(500)
             await self.send(StreamClosed(stream_id=self.stream_id))
         else:
             if message["type"] == "http.response.start" and self.state == ASGIHTTPState.REQUEST:
@@ -156,3 +158,17 @@ class HTTPStream:
                         await self.send(StreamClosed(stream_id=self.stream_id))
             else:
                 raise UnexpectedMessage(self.state, message["type"])
+
+    async def _send_error_response(self, status_code: int) -> None:
+        await self.send(
+            Response(
+                stream_id=self.stream_id,
+                headers=[(b"content-length", b"0"), (b"connection", b"close")],
+                status_code=status_code,
+            )
+        )
+        await self.send(EndBody(stream_id=self.stream_id))
+        self.state = ASGIHTTPState.CLOSED
+        await self.config.log.access(
+            self.scope, {"status": status_code, "headers": []}, time() - self.start_time
+        )
